from allauth.account.models import EmailAddress
from allauth.account.signals import user_signed_up
from allauth.account.utils import send_email_confirmation
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.contrib.auth.models import User
from .models import Profile

# @receiver(post_save, sender=User)
# def create_user_profile(sender, instance, created, **kwargs):
#     if created:
#         Profile.objects.create(user=instance)
#
# @receiver(post_save, sender=User)
# def save_user_profile(sender, instance, **kwargs):
#     instance.profile.save()

@receiver(user_signed_up)
def user_signed_up_callback(request, user, **kwargs):
    send_email_confirmation(request, user)

@receiver(post_save, sender=User)
def user_postsave(sender, instance, created, **kwargs):
    user = instance

    # add profile if user is created
    if created:
        Profile.objects.create(
            user=user,
        )
    # else:
    #     # update allauth emailaddress if exists
    #     try:
    #         email_address = EmailAddress.objects.get_primary(user)
    #         if email_address.email != user.email:
    #             email_address.email = user.email
    #             email_address.verified = False
    #             email_address.save()
    #     except:
    #         # if allauth emailaddress doesn't exist create one
    #         EmailAddress.objects.create(
    #             user=user,
    #             email=user.email,
    #             primary=True,
    #             verified=False
    #         )



@receiver(pre_save, sender=User)
def user_presave(sender, instance, **kwargs):
    if instance.username:
        instance.username = instance.username.lower()


from django.contrib.auth.models import User
from django.db import models
from django.templatetags.static import static


class Profile(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    profile_pic = models.ImageField( upload_to='avatars/', null=True, blank=True,)
    display_name = models.CharField(max_length=30, blank=True)
    bio = models.TextField(max_length=500, blank=True)
    location = models.CharField(max_length=30, blank=True)
    birth_date = models.DateField(null=True, blank=True)
    website = models.URLField(blank=True)
    facebook = models.URLField(blank=True)
    twitter = models.URLField(blank=True)
    instagram = models.URLField(blank=True)
    linkedin = models.URLField(blank=True)

    def __str__(self):
        return f'{self.user}'

    @property
    def name(self):
        if self.display_name:
            return self.display_name
        else:
            return self.user.username

    @property
    def avatar(self):
        if self.profile_pic:
            return self.profile_pic.url
        else:
            avatar = static('images/avatar.svg')
            return avatar



from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User

from .models import Profile


class CustomUserCreationForm(UserCreationForm):
    class Meta:
        model = User
        fields = ['username', 'email', 'password1', 'password2']

        widgets = {
            'username': forms.TextInput(attrs={'class': 'form-control'}),
            'email': forms.EmailInput(attrs={'class': 'form-control'}),
            'password1': forms.PasswordInput(attrs={'class': 'form-control'}),
            'password2': forms.PasswordInput(attrs={'class': 'form-control'}),

        }
        help_texts = {
            'username': None,
            'email': None,
            'password1': None,
            'password2': None,

        }
        error_texts = {
            'username': None,
            'email': None,
            'password1': None,
            'password2': None,

        }

class UserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('username', 'email', 'first_name', 'last_name')
        widgets = {
            'username': forms.TextInput(attrs={'class': 'form-control'}),
            'email': forms.EmailInput(attrs={'class': 'form-control'}),
            'first_name': forms.TextInput(attrs={'class': 'form-control'}),
            'last_name': forms.TextInput(attrs={'class': 'form-control'}),
        }

class ProfileForm(forms.ModelForm):
    class Meta:
        model = Profile
        fields = ['profile_pic','display_name', 'bio', 'location', 'birth_date', 'website', 'facebook', 'twitter', 'instagram', 'linkedin']

        widgets = {

            'profile_pic': forms.FileInput(attrs={'class': 'form-control'}),
            'display_name': forms.TextInput(attrs={'class': 'form-control'}),  # 'class': 'form-control
            'bio': forms.Textarea(attrs={'class': 'form-control'}),
            'location': forms.TextInput(attrs={'class': 'form-control'}),
            'birth_date': forms.DateInput(attrs={'class': 'form-control'}),
            'website': forms.URLInput(attrs={'class': 'form-control'}),
            'facebook': forms.URLInput(attrs={'class': 'form-control'}),
            'twitter': forms.URLInput(attrs={'class': 'form-control'}),
            'instagram': forms.URLInput(attrs={'class': 'form-control'}),
            'linkedin': forms.URLInput(attrs={'class': 'form-control'}),
        }


# Generated by Django 5.0.7 on 2024-07-19 16:37

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('profile_pic', models.ImageField(blank=True, null=True, upload_to='avatars/')),
                ('display_name', models.CharField(blank=True, max_length=30)),
                ('bio', models.TextField(blank=True, max_length=500)),
                ('location', models.CharField(blank=True, max_length=30)),
                ('birth_date', models.DateField(blank=True, null=True)),
                ('website', models.URLField(blank=True)),
                ('facebook', models.URLField(blank=True)),
                ('twitter', models.URLField(blank=True)),
                ('instagram', models.URLField(blank=True)),
                ('linkedin', models.URLField(blank=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]




from django.conf.urls.static import static
from django.conf import settings
from django.urls import path

from . import views

from .views import (

    ProfileView, Profile_edit_View, profile_email_verify, profile_settings_view,
    # EditProfileView,
    # ChangePasswordView,
)


urlpatterns = [
    path('', ProfileView.as_view(), name="profile"),
    path('profile/', ProfileView.as_view(), name='profile'),
    path('onboarding/', ProfileView.as_view(), name="profile-onboarding"),
    path('profile/edit/', Profile_edit_View.as_view(), name='edit_profile'),
    path('emailverify/', profile_email_verify, name="profile-email-verify"),
    path('settings/', profile_settings_view, name="profile-settings"),
    path('@<username>/', ProfileView.as_view(), name="profile"),

    # path('profile/password/', views.change_password, name='change_password'),

]



from allauth.account.utils import send_email_confirmation
from django.contrib.auth import logout, login
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User
from django.contrib.auth.views import LoginView
from django.http import request, HttpResponseNotFound
from django.shortcuts import render, redirect, get_object_or_404
from django.urls import reverse_lazy, reverse
from django.utils.decorators import method_decorator
from django.views.generic import CreateView, View

from .forms import CustomUserCreationForm, ProfileForm, UserForm
from .models import Profile



class ProfileView(View):
    template_name = 'profile.html'
    # form_class = UserProfileForm(instance=request.user)

    def get(self, request,username=None, *args, **kwargs):
        if username:
            # profile = get_object_or_404(User, username=username).profile
            try:
                profile = User.objects.get(username=username).profile
            except User.DoesNotExist:
                return HttpResponseNotFound(render(request, '404.html'))
        else:
            if request.user.is_authenticated:
                profile = request.user.profile
            else:
                return redirect('account_login')
        return render(request, self.template_name, {'profile': profile})

@method_decorator(login_required, name='dispatch')
class Profile_edit_View(View):
    template_name = 'profile_edit.html'

    def get(self, request, *args, **kwargs):
        profile_form = ProfileForm(instance=request.user.profile)
        onboarding = False
        if request.path == reverse('profile-onboarding'):
            onboarding = True

        return render(request, self.template_name, {'form': profile_form, 'onboarding': onboarding})

    def post(self, request, *args, **kwargs):
        profile_form = ProfileForm(request.POST, request.FILES, instance=request.user.profile)
        if  profile_form.is_valid():
            profile_form.save()
            return redirect('profile')
        return render(request, self.template_name, {'form': profile_form})



@login_required
def profile_email_verify(request):
    send_email_confirmation(request, request.user)
    return redirect('profile-settings')

@login_required
def profile_settings_view(request):
    return render(request, 'profile_settings.html')

